<?php
require_once('admin/include/mysmarty.php');
require_once('admin/include/headers.php');
require_once('admin/include/locale.php');
require_once('admin/include/authenticate.php');
require_once('admin/include/imapannotatemore.class.php');

/**** Arrays ****/
$debug = false;
$errors = array();
$success_messages = array();
$valid_actions = array( 'update' );
$valid_kolab_annotations = array( 'contact', 'note', 'task', 'event', 
                                  'contact.default', 'note.default', 
                                  'task.default', 'event.default' );


/**** Menu ***/
$sidx = 'syncpref';
require_once('admin/include/menu.php');
$menuitems[$sidx]['selected'] = 'selected';


/**** Extract mailboxes from IMAP ****/
$kolab_imap = new IMAP_annotatemore( 'localhost', '143' );
$kolab_imap->connect( $auth->uid(), $auth->password() );
try{
  $mb = $kolab_imap->getMailboxes();
}catch( Exception $e ){
  array_push($errors, _($e->getMessage()) );
}



/**** Set posted folder annotations if necessary ****/ 
if( empty($errors) && array_key_exists('submitsyncprefs', $_REQUEST) ) {
  // Decode postvalues, since we encoded the foldernames. See below.
  $postvalues = array();
  foreach( $_POST as $key => $value ) {
    $key = rawurldecode( $key );
    $postvalues[$key] = $value;
  }

  // Set/Delete Annotations where necessary
  foreach( $mb as $mailbox ) {
    if( array_key_exists( $mailbox, $postvalues )  ) {
      try{
        $kolab_imap->setAnnotation( "/vendor/tarent/syncpref", 
                                    array("value.priv" => "syncme"), $mailbox );
      }catch( Exception $e ){
        array_push( $errors, _($e->getMessage()) );
      }
    }else {
      // Since we're showing only kolab-annotated folders,
      //   we also only set annotations for these folders.
      try{
        $kolab_anno  = $kolab_imap->getAnnotation("/vendor/kolab/folder-type",
                                                  "value.shared", $mailbox);
        if( !in_array( trim($kolab_anno), $valid_kolab_annotations ) ) continue;

        $tarent_anno = $kolab_imap->getAnnotation( "/vendor/tarent/syncpref",
                                                   "value.priv", $mailbox);
        if( !empty($tarent_anno) ){
          $kolab_imap->setAnnotation( "/vendor/tarent/syncpref", 
                                      array("value.priv" => "NIL"), $mailbox );
        }
      }catch(Exceptions $e){
        array_push( $errors, _($e->getMessage()) );
      }
    }
  }
  if( empty($errors) ){
  	array_push( $success_messages, "Update was successful" );
  }
}



$imap_folders = array();
foreach( $mb as $mailbox ){
   $entry = array();
   
   /** 
    * (raw)urlencode will encode all special chars except for '-', '.' and '_'.
    *  Since PHP turns dots in incoming post-values to underscores, we have to 
    *  encode these, too. To make the output look consequent encoded we encode
    *  the minus as well.  
    */
   $entry['folder_id'] = str_replace(     '-', '%2D',
                           str_replace(   '_', '%5F', 
                             str_replace( '.', '%2E',
                               rawurlencode($mailbox) ) ) );
                               
   /** 
    * For presentation in the browser convert 
    * the modified UTF-7 encoding to UTF-8
    */                               
   $entry['full_name'] = mb_convert_encoding( $mailbox, "UTF-8", "UTF7-IMAP" );

   try{
     $kolab_anno = $kolab_imap->getAnnotation( "/vendor/kolab/folder-type",
                                               "value.shared", $mailbox );
   }catch( Exception $e ){
     array_push( $errors, _($e->getMessage()) );
     continue;
   }
   if( !in_array( trim($kolab_anno), $valid_kolab_annotations  ) ) continue;
   $entry['kolab_annotation'] = $kolab_anno;

   try{
     $tarent_anno = $kolab_imap->getAnnotation( "/vendor/tarent/syncpref",
                                                "value.priv", $mailbox );
   }catch(Exception $e){
     array_push( $errors, _($e->getMessage()) );
     continue;
   }
   $entry['tarent_annotation'] = $tarent_anno;
   if( $tarent_anno == 'syncme' ){
     $entry['enabled'] = true;
   }

   array_push( $imap_folders, $entry );
}



/**** Insert into template and output ***/
$smarty = new MySmarty();
$smarty->assign( 'messages', $success_messages );
$smarty->assign( 'debug', $debug );
$smarty->assign( 'errors', $errors );
$smarty->assign( 'uid', $auth->uid() );
$smarty->assign( 'group', $auth->group() );
$smarty->assign( 'page_title', $menuitems[$sidx]['title'] );
$smarty->assign( 'imap_folders', $imap_folders );
$smarty->assign( 'menuitems', $menuitems );
$smarty->assign( 'submenuitems', 
				 array_key_exists( 'submenu', $menuitems[$sidx] ) 
				   ? $menuitems[$sidx]['submenu']
				   : array() 
			   );
$smarty->assign( 'maincontent', 'syncpref.tpl' );
$smarty->display('page.tpl');

?>
